Package: party
Title: A Laboratory for Recursive Partytioning
Date: 2026-03-25
Version: 1.3-20
Authors@R: c(person("Torsten", "Hothorn", role = c("aut", "cre"),
                    email = "Torsten.Hothorn@R-project.org",
                    comment = c(ORCID = "0000-0001-8301-0471")),
             person("Kurt", "Hornik", role = "aut",
	            email = "Kurt.Hornik@R-project.org",
                    comment = c(ORCID = "0000-0003-4198-9911")),
             person("Carolin", "Strobl", role = "aut",
	            email = "carolin.strobl@psychologie.uzh.ch",
                    comment = c(ORCID = "0000-0003-0952-3230")),
             person("Achim", "Zeileis", role = "aut",
	            email = "Achim.Zeileis@R-project.org",
                    comment = c(ORCID = "0000-0003-0918-3766")))
Description: A computational toolbox for recursive partitioning.
  The core of the package is ctree(), an implementation of
  conditional inference trees which embed tree-structured 
  regression models into a well defined theory of conditional
  inference procedures. This non-parametric class of regression
  trees is applicable to all kinds of regression problems, including
  nominal, ordinal, numeric, censored as well as multivariate response
  variables and arbitrary measurement scales of the covariates. 
  Based on conditional inference trees, cforest() provides an
  implementation of Breiman's random forests. The function mob()
  implements an algorithm for recursive partitioning based on
  parametric models (e.g. linear models, GLMs or survival
  regression) employing parameter instability tests for split
  selection. Extensible functionality for visualizing tree-structured
  regression models is available. The methods are described in
  Hothorn et al. (2006) <doi:10.1198/106186006X133933>,
  Zeileis et al. (2008) <doi:10.1198/106186008X319331> and 
  Strobl et al. (2007) <doi:10.1186/1471-2105-8-25>.
Depends: R (>= 3.0.0), methods, grid, stats, mvtnorm (>= 1.0-2),
        modeltools (>= 0.2-21), strucchange
LinkingTo: mvtnorm
Imports: survival (>= 2.37-7), coin (>= 1.1-0), zoo, sandwich (>=
        1.1-1)
Suggests: TH.data (>= 1.0-3), mlbench, colorspace, MASS, vcd, ipred,
        varImp, randomForest, lattice, AER, bibtex
LazyData: yes
License: GPL-2
URL: http://party.R-forge.R-project.org
NeedsCompilation: yes
Packaged: 2026-03-25 16:05:56 UTC; hothorn
Author: Torsten Hothorn [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8301-0471>),
  Kurt Hornik [aut] (ORCID: <https://orcid.org/0000-0003-4198-9911>),
  Carolin Strobl [aut] (ORCID: <https://orcid.org/0000-0003-0952-3230>),
  Achim Zeileis [aut] (ORCID: <https://orcid.org/0000-0003-0918-3766>)
Maintainer: Torsten Hothorn <Torsten.Hothorn@R-project.org>
Repository: CRAN
Date/Publication: 2026-03-26 06:10:03 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-28 22:57:10 UTC; unix
Archs: party.so.dSYM
